@@ -1,14 +1,23 @@
 # OBSS SAHI Tool
 # Code written by Fatih C Akyon, 2020.
 
+import logging
 import os
 import time
+import warnings
 from typing import Dict, List, Optional
 
 import numpy as np
 from tqdm import tqdm
 
-from sahi.postprocess.combine import NMSPostprocess, PostprocessPredictions, UnionMergePostprocess
+from sahi.postprocess.combine import (
+    GreedyNMMPostprocess,
+    LSNMSPostprocess,
+    NMMPostprocess,
+    NMSPostprocess,
+    PostprocessPredictions,
+)
+from sahi.postprocess.legacy.combine import UnionMergePostprocess
 from sahi.prediction import ObjectPrediction, PredictionResult
 from sahi.slicing import slice_image
 from sahi.utils.coco import Coco, CocoImage
@@ -21,6 +30,9 @@ MODEL_TYPE_TO_MODEL_CLASS_NAME = {
 }
 
 
+logger = logging.getLogger(__name__)
+
+
 def get_prediction(
     image,
     detection_model,
@@ -54,6 +66,9 @@ def get_prediction(
             object_prediction_list: a list of ObjectPrediction
             durations_in_seconds: a dict containing elapsed times for profiling
     """
+    if image_size is not None:
+        warnings.warn("Set 'image_size' at DetectionModel init.", DeprecationWarning)
+
     durations_in_seconds = dict()
 
     # read image as pil
@@ -72,15 +87,10 @@ def get_prediction(
         full_shape=full_shape,
     )
     object_prediction_list: List[ObjectPrediction] = detection_model.object_prediction_list
-    # filter out predictions with lower score
-    filtered_object_prediction_list = [
-        object_prediction
-        for object_prediction in object_prediction_list
-        if object_prediction.score.value > detection_model.confidence_threshold
-    ]
+
     # postprocess matching predictions
     if postprocess is not None:
-        filtered_object_prediction_list = postprocess(filtered_object_prediction_list)
+        object_prediction_list = postprocess(object_prediction_list)
 
     time_end = time.time() - time_start
     durations_in_seconds["postprocess"] = time_end
@@ -93,7 +103,7 @@ def get_prediction(
         )
 
     return PredictionResult(
-        image=image, object_prediction_list=filtered_object_prediction_list, durations_in_seconds=durations_in_seconds
+        image=image, object_prediction_list=object_prediction_list, durations_in_seconds=durations_in_seconds
     )
 
 
@@ -101,12 +111,12 @@ def get_sliced_prediction(
     image,
     detection_model=None,
     image_size: int = None,
-    slice_height: int = 256,
-    slice_width: int = 256,
+    slice_height: int = 512,
+    slice_width: int = 512,
     overlap_height_ratio: float = 0.2,
     overlap_width_ratio: float = 0.2,
     perform_standard_pred: bool = True,
-    postprocess_type: str = "UNIONMERGE",
+    postprocess_type: str = "GREEDYNMM",
     postprocess_match_metric: str = "IOS",
     postprocess_match_threshold: float = 0.5,
     postprocess_class_agnostic: bool = False,
@@ -122,23 +132,23 @@ def get_sliced_prediction(
         image_size: int
             Input image size for each inference (image is scaled by preserving asp. rat.).
         slice_height: int
-            Height of each slice.  Defaults to ``256``.
+            Height of each slice.  Defaults to ``512``.
         slice_width: int
-            Width of each slice.  Defaults to ``256``.
+            Width of each slice.  Defaults to ``512``.
         overlap_height_ratio: float
             Fractional overlap in height of each window (e.g. an overlap of 0.2 for a window
-            of size 256 yields an overlap of 51 pixels).
+            of size 512 yields an overlap of 102 pixels).
             Default to ``0.2``.
         overlap_width_ratio: float
             Fractional overlap in width of each window (e.g. an overlap of 0.2 for a window
-            of size 256 yields an overlap of 51 pixels).
+            of size 512 yields an overlap of 102 pixels).
             Default to ``0.2``.
         perform_standard_pred: bool
             Perform a standard prediction on top of sliced predictions to increase large object
             detection accuracy. Default: True.
         postprocess_type: str
             Type of the postprocess to be used after sliced inference while merging/eliminating predictions.
-            Options are 'UNIONMERGE' or 'NMS'. Default is 'UNIONMERGE'.
+            Options are 'NMM', 'GRREDYNMM' or 'NMS'. Default is 'GRREDYNMM'.
         postprocess_match_metric: str
             Metric to be used during object prediction matching after sliced prediction.
             'IOU' for intersection over union, 'IOS' for intersection over smaller area.
@@ -157,6 +167,9 @@ def get_sliced_prediction(
             object_prediction_list: a list of sahi.prediction.ObjectPrediction
             durations_in_seconds: a dict containing elapsed times for profiling
     """
+    if image_size is not None:
+        warnings.warn("Set 'image_size' at DetectionModel init.", DeprecationWarning)
+
     # for profiling
     durations_in_seconds = dict()
 
@@ -177,8 +190,14 @@ def get_sliced_prediction(
     durations_in_seconds["slice"] = time_end
 
     # init match postprocess instance
-    if postprocess_type == "UNIONMERGE":
-        postprocess = UnionMergePostprocess(
+    if postprocess_type in ["NMM"]:
+        postprocess = NMMPostprocess(
+            match_threshold=postprocess_match_threshold,
+            match_metric=postprocess_match_metric,
+            class_agnostic=postprocess_class_agnostic,
+        )
+    elif postprocess_type == "GREEDYNMM":
+        postprocess = GreedyNMMPostprocess(
             match_threshold=postprocess_match_threshold,
             match_metric=postprocess_match_metric,
             class_agnostic=postprocess_class_agnostic,
@@ -189,13 +208,29 @@ def get_sliced_prediction(
             match_metric=postprocess_match_metric,
             class_agnostic=postprocess_class_agnostic,
         )
+    elif postprocess_type == "LSNMS":
+        postprocess = LSNMSPostprocess(
+            match_threshold=postprocess_match_threshold,
+            match_metric=postprocess_match_metric,
+            class_agnostic=postprocess_class_agnostic,
+        )
+    elif postprocess_type == "UNIONMERGE":
+        # sahi v0.8.16 compatibility
+        logger.warning("'UNIONMERGE' is deprecated, use 'GREEDYNMM' instead.")
+        postprocess = UnionMergePostprocess(
+            match_threshold=postprocess_match_threshold,
+            match_metric=postprocess_match_metric,
+            class_agnostic=postprocess_class_agnostic,
+        )
     else:
-        raise ValueError(f"postprocess_type should be one of ['UNIOUNMERGE', 'NMS'] but given as {postprocess_type}")
+        raise ValueError(
+            f"postprocess_type should be one of ['NMS', 'NMM', 'GREEDYNMM'] but given as {postprocess_type}"
+        )
 
     # create prediction input
     num_group = int(num_slices / num_batch)
     if verbose == 1 or verbose == 2:
-        print("Number of slices:", num_slices)
+        tqdm.write(f"Performing prediction on {num_slices} number of slices.")
     object_prediction_list = []
     # perform sliced prediction
     for group_ind in range(num_group):
@@ -216,9 +251,12 @@ def get_sliced_prediction(
                 slice_image_result.original_image_width,
             ],
         )
-        object_prediction_list.extend(prediction_result.object_prediction_list)
+        # convert sliced predictions to full predictions
+        for object_prediction in prediction_result.object_prediction_list:
+            if object_prediction:  # if not empty
+                object_prediction_list.append(object_prediction.get_shifted_object_prediction())
+    # perform standard prediction
     if num_slices > 1 and perform_standard_pred:
-        # perform standard prediction
         prediction_result = get_prediction(
             image=image,
             detection_model=detection_model,
@@ -229,14 +267,6 @@ def get_sliced_prediction(
         )
         object_prediction_list.extend(prediction_result.object_prediction_list)
 
-    # remove empty predictions
-    object_prediction_list = [object_prediction for object_prediction in object_prediction_list if object_prediction]
-    # convert sliced predictions to full predictions
-    full_object_prediction_list = []
-    for object_prediction in object_prediction_list:
-        full_object_prediction = object_prediction.get_shifted_object_prediction()
-        full_object_prediction_list.append(full_object_prediction)
-
     time_end = time.time() - time_start
     durations_in_seconds["prediction"] = time_end
 
@@ -253,10 +283,10 @@ def get_sliced_prediction(
         )
 
     # merge matching predictions
-    full_object_prediction_list = postprocess(full_object_prediction_list)
+    object_prediction_list = postprocess(object_prediction_list)
 
     return PredictionResult(
-        image=image, object_prediction_list=full_object_prediction_list, durations_in_seconds=durations_in_seconds
+        image=image, object_prediction_list=object_prediction_list, durations_in_seconds=durations_in_seconds
     )
 
 
@@ -272,11 +302,11 @@ def predict(
     no_standard_prediction: bool = False,
     no_sliced_prediction: bool = False,
     image_size: int = None,
-    slice_height: int = 256,
-    slice_width: int = 256,
+    slice_height: int = 512,
+    slice_width: int = 512,
     overlap_height_ratio: float = 0.2,
     overlap_width_ratio: float = 0.2,
-    postprocess_type: str = "UNIONMERGE",
+    postprocess_type: str = "GREEDYNMM",
     postprocess_match_metric: str = "IOS",
     postprocess_match_threshold: float = 0.5,
     postprocess_class_agnostic: bool = False,
@@ -319,20 +349,20 @@ def predict(
         image_size: int
             Input image size for each inference (image is scaled by preserving asp. rat.).
         slice_height: int
-            Height of each slice.  Defaults to ``256``.
+            Height of each slice.  Defaults to ``512``.
         slice_width: int
-            Width of each slice.  Defaults to ``256``.
+            Width of each slice.  Defaults to ``512``.
         overlap_height_ratio: float
             Fractional overlap in height of each window (e.g. an overlap of 0.2 for a window
-            of size 256 yields an overlap of 51 pixels).
+            of size 512 yields an overlap of 102 pixels).
             Default to ``0.2``.
         overlap_width_ratio: float
             Fractional overlap in width of each window (e.g. an overlap of 0.2 for a window
-            of size 256 yields an overlap of 51 pixels).
+            of size 512 yields an overlap of 102 pixels).
             Default to ``0.2``.
         postprocess_type: str
             Type of the postprocess to be used after sliced inference while merging/eliminating predictions.
-            Options are 'UNIONMERGE' or 'NMS'. Default is 'UNIONMERGE'.
+            Options are 'NMM', 'GRREDYNMM' or 'NMS'. Default is 'GRREDYNMM'.
         postprocess_match_metric: str
             Metric to be used during object prediction matching after sliced prediction.
             'IOU' for intersection over union, 'IOS' for intersection over smaller area.
@@ -358,7 +388,8 @@ def predict(
             Can be specified as 'jpg' or 'png'
         verbose: int
             0: no print
-            1: print slice/prediction durations, number of slices, model loading/file exporting durations
+            1: print slice/prediction durations, number of slices
+            2: print model loading/file exporting durations
     """
     # assert prediction type
     assert (
@@ -406,6 +437,7 @@ def predict(
         category_mapping=model_category_mapping,
         category_remapping=model_category_remapping,
         load_at_init=False,
+        image_size=image_size,
     )
     detection_model.load_model()
     time_end = time.time() - time_start
@@ -414,7 +446,7 @@ def predict(
     # iterate over source images
     durations_in_seconds["prediction"] = 0
     durations_in_seconds["slice"] = 0
-    for ind, image_path in enumerate(tqdm(image_path_list)):
+    for ind, image_path in enumerate(tqdm(image_path_list, "Performing inference on images")):
         # get filename
         if os.path.isdir(source):  # preserve source folder structure in export
             relative_filepath = image_path.split(source)[-1]
@@ -431,7 +463,6 @@ def predict(
             prediction_result = get_sliced_prediction(
                 image=image_path,
                 detection_model=detection_model,
-                image_size=image_size,
                 slice_height=slice_height,
                 slice_width=slice_width,
                 overlap_height_ratio=overlap_height_ratio,
@@ -441,7 +472,7 @@ def predict(
                 postprocess_match_metric=postprocess_match_metric,
                 postprocess_match_threshold=postprocess_match_threshold,
                 postprocess_class_agnostic=postprocess_class_agnostic,
-                verbose=verbose,
+                verbose=1 if verbose else 0,
             )
             object_prediction_list = prediction_result.object_prediction_list
             durations_in_seconds["slice"] += prediction_result.durations_in_seconds["slice"]
@@ -450,7 +481,6 @@ def predict(
             prediction_result = get_prediction(
                 image=image_path,
                 detection_model=detection_model,
-                image_size=image_size,
                 shift_amount=[0, 0],
                 full_shape=None,
                 postprocess=None,
@@ -543,8 +573,11 @@ def predict(
         save_path = str(save_dir / "result.json")
         save_json(coco_json, save_path)
 
+    if export_visual or export_pickle or export_crop or dataset_json_path is not None:
+        print(f"Prediction results are sucessfully exported to {save_dir}")
+
     # print prediction duration
-    if verbose == 1:
+    if verbose == 2:
         print(
             "Model loaded in",
             durations_in_seconds["model_load"],
@@ -585,7 +618,7 @@ def predict_fiftyone(
     slice_width: int = 256,
     overlap_height_ratio: float = 0.2,
     overlap_width_ratio: float = 0.2,
-    postprocess_type: str = "UNIONMERGE",
+    postprocess_type: str = "GREEDYNMM",
     postprocess_match_metric: str = "IOS",
     postprocess_match_threshold: float = 0.5,
     postprocess_class_agnostic: bool = False,
@@ -633,7 +666,10 @@ def predict_fiftyone(
             Default to ``0.2``.
         postprocess_type: str
             Type of the postprocess to be used after sliced inference while merging/eliminating predictions.
-            Options are 'UNIONMERGE' or 'NMS'. Default is 'UNIONMERGE'.
+            Options are 'NMM', 'GRREDYNMM' or 'NMS'. Default is 'GRREDYNMM'.
+        postprocess_match_metric: str
+            Metric to be used during object prediction matching after sliced prediction.
+            'IOU' for intersection over union, 'IOS' for intersection over smaller area.
         postprocess_match_metric: str
             Metric to be used during object prediction matching after sliced prediction.
             'IOU' for intersection over union, 'IOS' for intersection over smaller area.
@@ -670,6 +706,7 @@ def predict_fiftyone(
         category_mapping=model_category_mapping,
         category_remapping=model_category_remapping,
         load_at_init=False,
+        image_size=image_size,
     )
     detection_model.load_model()
     time_end = time.time() - time_start
@@ -687,15 +724,14 @@ def predict_fiftyone(
                 prediction_result = get_sliced_prediction(
                     image=sample.filepath,
                     detection_model=detection_model,
-                    image_size=image_size,
                     slice_height=slice_height,
                     slice_width=slice_width,
                     overlap_height_ratio=overlap_height_ratio,
                     overlap_width_ratio=overlap_width_ratio,
                     perform_standard_pred=not no_standard_prediction,
                     postprocess_type=postprocess_type,
-                    postprocess_match_metric=postprocess_match_metric,
                     postprocess_match_threshold=postprocess_match_threshold,
+                    postprocess_match_metric=postprocess_match_metric,
                     postprocess_class_agnostic=postprocess_class_agnostic,
                     verbose=verbose,
                 )
@@ -705,7 +741,6 @@ def predict_fiftyone(
                 prediction_result = get_prediction(
                     image=sample.filepath,
                     detection_model=detection_model,
-                    image_size=image_size,
                     shift_amount=[0, 0],
                     full_shape=None,
                     postprocess=None,
